<?php

use Ballen\Distical\Calculator as DistanceCalculator;
use Ballen\Distical\Entities\LatLong;

class Place extends DataObject
{

    private static $db = [
        'Name' => 'Varchar(255)',
        'Website' => 'Varchar(255)',
        'Active' => 'Boolean',
        'Featured' => 'Boolean',
	    'Price' => 'Currency'
    ];

    private static $has_many = [
        'Galleries' => 'Gallery'
    ];

    private static $many_many = [
        'Categories' => 'PlaceCategory',
        'Properties' => 'Property'
    ];


    public function getCMSFields()
    {
        $fields = parent::getCMSFields();
        $fields->removeByName('Properties');
        $fields->removeByName('PlacesPageID');
        $fields->addFieldToTab('Root.Categories', new PickerField(
            'Categories',
            'Select categories',
            $this->Categories(),
            'Select Categories'
        ));
        return $fields;
    }


	public function getCMSValidator()
	{
		return new RequiredFields(['Name']);
	}


    protected function onBeforeWrite()
    {
	    if ($this->hasExtension('ContentExtension')) {
		    if (isset($_REQUEST['removeFeaturedImage']) && $_REQUEST['removeFeaturedImage']) {
			    $this->ImageID = 0;
		    }
	    }
	    if ($this->hasExtension('LocationExtension')) {
		    if (isset($_REQUEST['removeMapPin']) && $_REQUEST['removeMapPin']) {
			    $this->MapPinID = 0;
		    }
	    }
        $this->setModelSlug();
        parent::onBeforeWrite();
    }


    protected function onAfterWrite()
    {
        $place = Place::get()->byID($this->ID);
        Bootstrap::addRequiredGalleries(REQUIRED_PROPERTY_GALLERIES, $place);
        $this->addPropertiesWithinRadius();
        parent::onAfterWrite();
    }

    private function addPropertiesWithinRadius()
    {
        if ($this->Latitude && $this->Longitude) {
            $placeCoordinates = new LatLong($this->Latitude, $this->Longitude);
            $place = Place::get()->byID($this->ID);
            $properties = Property::get();
            foreach ($properties as $property) {
                $place->Properties()->remove($property);
                if ($property->Latitude && $property->Longitude) {
                    $propertyCoordinates = new LatLong($property->Latitude, $property->Longitude);
                    $distance = new DistanceCalculator($placeCoordinates, $propertyCoordinates);
                    $radius = $property->PlaceRadius ? $property->PlaceRadius : 50;
                    if ($distance->get()->asKilometres() <= $radius) {
                        $place->Properties()->add($property);
                    }
                }
            }
        }
    }


	private function setModelSlug()
	{
		if (!$this->Slug) {
			$this->Slug = strtolower(trim(preg_replace('/[^A-Za-z0-9-]+/', '-', $this->Name)));
		}
	}


	public function getJson()
	{
		$categories = [];
		foreach ($this->Categories() as $category) {
			$categories[] = ['name' => $category->Name];
		}
		$data = [
			'name' => $this->Name,
			'Website' => $this->Website,
			'active' => $this->Active,
			'categories' => $categories,
			'link' => $this->Link(),
			'location' => [
				'address1' => $this->Address1,
				'address2' => $this->Address2,
				'city' => $this->City,
				'postcode' => $this->Postcode,
				'state' => $this->State,
				'country' => $this->Country,
				'directions' => $this->Directions,
				'latitude' => $this->Latitude,
				'longitude' => $this->Longitude,
				'region' => $this->Region()->Name,
				'regionSlug' => $this->Region()->Slug,
				'regionShortName' => $this->Region()->ShortName
			],
			'content' => [
				'excerpt' => $this->Excerpt,
				'content' => $this->Content
			],
			'contact' => [
				'email' => $this->Email,
				'phone' => $this->Phone
			]
		];
		return json_encode($data, JSON_NUMERIC_CHECK);
	}


	public function Link()
	{
		$placesPage = PlacesPage::get()->first();
		return $placesPage->Link($this->Region()->Slug . '/' . $this->Slug);
	}

	public function AbsoluteLink()
	{
		return Director::absoluteURL($this->Link());
	}

	public function canView($member = null)
	{
		if (parent::canView($member)) {
			return true;
		}
		if (!$this->Active) {
			return false;
		}
		if (!$this->ShowInSiteMap) {
			return false;
		}
		return true;
	}

}